# ----------------------------------------------------------------------------
#
#  Copyright (C) 2008-2015 Fons Adriaensen <fons@linuxaudio.org>
#    
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http:#www.gnu.org/licenses/>.
#
# ----------------------------------------------------------------------------


from jacktools import jackclient_ext


class JackClient() :

    """
    Abstract base class
    """

    def __init__(self, jclient) :
        """
        For use by derived classes only !!
        """
        self._jclient = jclient


    def get_jack_info(self) :
        """
        Return the tuple (jack client name, sample rate, period size).
        """
        return jackclient_ext.get_jack_info ((self._jclient,))


    def create_input(self, ind, name) :
        """
        Create an input port with index 'ind' and name 'name'.
        Indices start at zero.
        """
        return jackclient_ext.create_inp_port ((self._jclient, ind, name))


    def delete_input(self, ind) :
        """
        Delete the input port with index 'ind', or all input
        ports if 'ind' == -1.
        """
        return jackclient_ext.delete_inp_port ((self._jclient, ind))


    def connect_input(self, ind, srce) :
        """
        Connect the input with index 'ind' to 'srce'.
        """
        return jackclient_ext.connect_inp_port ((self._jclient, ind, srce))


    def disconn_input (self, ind, srce = None) :
        """
        Disconnect an input port, or all input ports if 'ind' == -1.
        In the first case, if 'srce' is not None, only the named
        source is disconnected.
        """
        return jackclient_ext.disconn_inp_port ((self._jclient, ind))


    def create_output(self, ind, name) :
        """
        Create an output port with index 'ind' and name 'name'.
        Indices start at zero.
        """
        return jackclient_ext.create_out_port ((self._jclient, ind, name))

    
    def delete_output(self, ind) :
        """
        Delete the input port with index 'ind', or all input
        ports if 'ind' == -1.
        """
        return jackclient_ext.delete_out_port ((self._jclient, ind))


    def connect_output(self, ind, dest) :
        """
        Connect the output with index 'ind' to 'dest'.
        """
        return jackclient_ext.connect_out_port ((self._jclient, ind, dest))
        

    def disconn_output (self, ind, dest = None) :
        """
        Disconnect an output port, or all output ports if 'ind' == -1.
        In the first case, if 'dest' is not None, only the named
        destination is disconnected.
        """
        return jackclient_ext.disconn_out_port ((self._jclient, ind))


