# ----------------------------------------------------------------------------
#
#  Copyright (C) 2015 Fons Adriaensen <fons@linuxaudio.org>
#    
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http:#www.gnu.org/licenses/>.
#
# ----------------------------------------------------------------------------


from array import array
from jacktools.jackclient import JackClient
from jacktools import jacknmeter_ext


class JackNmeter(JackClient) :

    """
    Level and noise measurement according
    to various standards.
    """

    INITIAL = 0
    PROCESS = 3
    FAILED  = -1
    ZOMBIE  = -2

    FIL_NONE   = 0
    FIL_ENB20K = 1
    FIL_IEC_A  = 2
    FIL_IEC_C  = 3
    FIL_ITU468 = 4
    FIL_DOLBY  = 5
    
    DET_NONE      = 0
    DET_RMS       = 1
    DET_RMS_SLOW  = 2
    DET_VUM       = 3
    DET_VUM_SLOW  = 4
    DET_ITU468    = 5

    
    def __init__(self, nchan, client_name, server_name = None):
        """
        Create a new JackNmeter instance.

        The optional 'server_name' allows to select between running
        Jack servers. The result should be checked using get_levels().
        """
        self._level = array ('f', (0 for i in range (nchan)))
        self._jnmeter = jacknmeter_ext.jnmeter_caps ((self, client_name, server_name, nchan, self._level))
        super().__init__(jacknmeter_ext.jclient_caps ((self._jnmeter,)))
        self.set_dcfilt (-1, 0)
        self.set_filter (-1, JackNmeter.FIL_NONE)
        self.set_detect (-1, JackNmeter.DET_RMS)
        

    def set_dcfilt(self, chan, onoff) :
        """
        Set DC filter on or off for chan, or for all channels if chan < 0.
        """
        return jacknmeter_ext.set_option ((self._jnmeter, chan, 0, onoff))


    def set_filter(self, chan, ftype) :
        """
        Set weighting filter for chan, or for all channels if chan < 0.
        """
        return jacknmeter_ext.set_option ((self._jnmeter, chan, 1, ftype))


    def set_detect(self, chan, dtype) :
        """
        Set detector type for chan, or for all channels if chan < 0.
        """
        return jacknmeter_ext.set_option ((self._jnmeter, chan, 2, dtype))


    def get_levels(self) :
        """
        Update the 'level' array, and return state.
        """
        return (jacknmeter_ext.get_levels ((self._jnmeter,)), self._level)


